// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		in vec2 uv0;
		in vec2 uv1;
	#endif
#endif

#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include "uniformInterFrameConstants.h"
#include "util.h"

varying hmp vec3 cpos;
varying hmp vec3 wpos;
varying vec4 color;
varying float farrender;
varying float farchunk;

#ifdef UNDERWATER
varying float fogr;
#endif

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

vec3 Film(vec3 x)
{
	 float a = 3.15;
	 float b = 0.02;
	 float c = 2.43;
	 float d = 0.59;
	 float e = 0.14;
	return clamp((x*(a*x+b))/(x*(c*x+d)+e),0.05,1.0);
}

#include "lib/s_lib.h"
#define rain clamp((1.0-pow(FOG_CONTROL.y+.1,10.)),.0,1.)*uv1.y

vec3 calclight(vec3 c,float strength){
	vec3 lcolor = mix(vec3(dot(vec3(7.,4.,0)*strength,vec3(1.))),vec3(7.,4.,0)*strength,1.1);
	c += lcolor*pow(uv1.x*1.1,5.);
return c;
}

vec4 calcWater(vec4 c,vec4 btex,vec3 N,vec2 tt,vec2 ft,float rim){
	vec3 nw = normalize(vec3(calcWave(cpos.xz*5.)*0.6,calcWave(cpos.xz*5.)*0.65,1.));
	vec4 wcolor = vec4(c.rgb,.2);
	vec3 npos = normalize(wpos*vec3(1,1,6));
	float sm = (1.-rain)*max(0.,1.-length(npos.zy))*13.,slook = (1.-FOG_COLOR.b)*clamp((FOG_COLOR.r-.15)*1.25,0.,1.);
	float hmap = pow(dot(nw,normalize(vec3(.8,-1.,.8))),4.);
	wcolor.rgb = mix(wcolor.rgb,calclight(wcolor.rgb,mix(.1,0.,1.-tt.x)),(uv1.x+hmap*2.));
		c *= .5;
		c = mix(wcolor+hmap*btex.r,FOG_COLOR+min(1.2*slook*ft.y*3.5,1.)*(sm*hmap)*smoothstep(0.,0.8,uv1.y),FOG_CONTROL.x==0.?-rim*.5:rim*.8);
		c.a += pow(farchunk,5.);
return c;
}

vec3 calcFog(vec3 c,vec2 ft){
	vec3 norpos = normalize(wpos);
	float sunamount = max(0.,1.-length(norpos.zy)), sunlook = (1.-FOG_COLOR.b)*clamp((FOG_COLOR.r-.15)*1.25,0.,1.);
	vec3 fogcol = mix(FOG_COLOR.rgb,FOG_COLOR.rgb*1.5,sunamount*min(1.2*sunlook*ft.y*4.,1.));
return mix(c,fogcol,pow(farrender*.9,1.8));
}

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 
#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
	vec4 lm = texture2D_AA(TEXTURE_1, vec2(0,1));
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
	vec4 lm = texture2D(TEXTURE_1, vec2(0,1));
#endif

	vec4 btex = texture2D(TEXTURE_1,uv1);
	bool nether = ((FOG_COLOR.r > FOG_COLOR.b) && (FOG_COLOR.r < 0.5) && (FOG_COLOR.b < 0.2));
	bool water = (color.a < 0.95 && color.a > 0.05);
	hmp vec2 ttime = vec2(max(1.-min(pow(lm.r*1.8,5.),1.),.0),pow(clamp((1.-pow(lm.r,5.))*1.3,.0,1.),8.))*uv1.y;
	vec2 ftime = vec2(pow(max(min(1.0-FOG_COLOR.r*1.5,1.),.0),1.2),pow(max(min(1.0-FOG_COLOR.r*1.2,1.),.0),.5));

#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

	vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= btex;
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif

//reduce default ambient occlusion
if(color.g > color.b && color.a != 0.){
	diffuse.rgb *= mix(normalize(inColor.rgb),inColor.rgb,0.5);
}else{  
	diffuse.rgb *= color.a ==0.?inColor.rgb:sqrt(inColor.rgb);
}

#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

#ifdef FANCY
	vec3 N = normalize(cross(dFdx(cpos.xyz),dFdy(cpos.xyz)));
#endif
	float smap = 0.,rim = pow(smoothstep(0.,1.,1.-dot(N,normalize(-wpos))),2.);
#if !defined(ALPHA_TEST)
	smap = mix(smap,1.,max(.0,max(N.x,-N.x)));
#endif 
	smap= mix(smap,bool(nether)?0.:0.9,1.-min(max((uv1.y-0.853)*100.0,0.0),1.0));
	diffuse.rgb = (diffuse.rgb) + 
(diffuse.r + diffuse.g + diffuse.b)*(-.06);
	diffuse.rgb = calclight(diffuse.rgb,mix(0.01,0.03,1.-uv1.y));

#if !defined(SEASONS) && !defined(ALPHA_TEST)
if(bool(water)==true)diffuse = calcWater(diffuse,btex,N,ttime,ftime,rim);
#endif

//tone
diffuse.rgb = Film(diffuse.rgb);

#ifdef UNDERWATER
if(bool(water)==false)diffuse = vec4(FOG_COLOR.rgb,.0)*diffuse + max(0.,calcWave(cpos.xz*5.5))*uv1.y*diffuse*1.3;
	diffuse.rgb += (1.-uv1.y)*pow(uv1.x*1., 4.)*vec3(1,1,1)*diffuse.rgb;
	diffuse.rgb = mix(diffuse.rgb,FOG_COLOR.rgb,pow(fogr,7.));
#endif
	diffuse.rgb *= mix(1.,.6,smap*(1.-rain)*(1.-max(0.,2.*uv1.x-0.8)));
	diffuse.rgb = calcFog(diffuse.rgb,ftime);

	gl_FragColor = diffuse;
#endif // BYPASS_PIXEL_SHADER
}
